declare class MessageHeader {
    messageLength: number;
    requestID: number;
    responseTo: number;
    opCode: number;
    constructor(values: Pick<MessageHeader, 'messageLength' | 'requestID' | 'responseTo' | 'opCode'>);
    static Read(buf: Uint8Array): Promise<{
        msg: MessageHeader;
        readBytes: number;
    }>;
}
declare type OpContents = OpMsg | OpReply | OpUpdate | OpInsert | OpQuery | OpGetMore | OpDelete | OpKillCursors | OpCompressed | OpUnknown;
declare class BSONBuffer {
    data: any;
    constructor(buf: Uint8Array);
}
declare class OpMsgBodySection {
    kind: "Body";
    body: BSONBuffer;
    constructor(body: BSONBuffer);
}
declare class OpMsgSecurityTokenSection {
    kind: "SecurityToken";
    body: BSONBuffer;
    constructor(body: BSONBuffer);
}
declare class OpMsgDocumentSequenceSection {
    kind: "DocumentSequence";
    docSequenceId: string;
    objects: BSONBuffer[];
    constructor(docSequenceId: string, objects: BSONBuffer[]);
}
declare class OpMsgUnknownSection {
    kind: "Unknown";
    kindByte: number;
    constructor(kindByte: number);
}
declare class OpMsg {
    opCode: "OP_MSG";
    flagBits: number;
    sections: (OpMsgBodySection | OpMsgDocumentSequenceSection | OpMsgSecurityTokenSection | OpMsgUnknownSection)[];
    checksum: number | null;
    constructor(values: Pick<OpMsg, 'flagBits' | 'sections' | 'checksum'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpMsg;
        readBytes: number;
    }>;
}
declare class OpUpdate {
    opCode: "OP_UPDATE";
    fullCollectionName: string;
    flags: number;
    selector: BSONBuffer;
    update: BSONBuffer;
    constructor(values: Pick<OpUpdate, 'fullCollectionName' | 'flags' | 'selector' | 'update'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpUpdate;
        readBytes: number;
    }>;
}
declare class OpInsert {
    opCode: "OP_INSERT";
    fullCollectionName: string;
    flags: number;
    documents: BSONBuffer[];
    constructor(values: Pick<OpInsert, 'fullCollectionName' | 'flags' | 'documents'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpInsert;
        readBytes: number;
    }>;
}
declare class OpQuery {
    opCode: "OP_QUERY";
    fullCollectionName: string;
    flags: number;
    numberToSkip: number;
    numberToReturn: number;
    query: BSONBuffer;
    returnFieldsSelector: BSONBuffer | null;
    constructor(values: Pick<OpQuery, 'fullCollectionName' | 'flags' | 'numberToSkip' | 'numberToReturn' | 'query' | 'returnFieldsSelector'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpQuery;
        readBytes: number;
    }>;
}
declare class OpGetMore {
    opCode: "OP_GET_MORE";
    fullCollectionName: string;
    numberToReturn: number;
    cursorId: string;
    constructor(values: Pick<OpGetMore, 'fullCollectionName' | 'numberToReturn' | 'cursorId'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpGetMore;
        readBytes: number;
    }>;
}
declare class OpDelete {
    opCode: "OP_DELETE";
    fullCollectionName: string;
    flags: number;
    selector: BSONBuffer;
    constructor(values: Pick<OpDelete, 'fullCollectionName' | 'flags' | 'selector'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpDelete;
        readBytes: number;
    }>;
}
declare class OpKillCursors {
    opCode: "OP_KILL_CURSORS";
    cursorIds: string[];
    constructor(values: Pick<OpKillCursors, 'cursorIds'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpKillCursors;
        readBytes: number;
    }>;
}
declare class OpCompressed {
    opCode: "OP_COMPRESSED";
    originalOpcode: number;
    compressorId: string;
    contents: OpContents | Uint8Array;
    constructor(values: Pick<OpCompressed, 'originalOpcode' | 'compressorId' | 'contents'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpCompressed;
        readBytes: number;
    }>;
}
declare class OpReply {
    opCode: "OP_REPLY";
    flags: number;
    cursorId: string;
    startingFrom: number;
    numberReturned: number;
    documents: BSONBuffer[];
    constructor(values: Pick<OpReply, 'flags' | 'cursorId' | 'startingFrom' | 'numberReturned' | 'documents'>);
    static Read(buf: Uint8Array, length: number): Promise<{
        msg: OpReply;
        readBytes: number;
    }>;
}
declare class OpUnknown {
    opCode: "OP_UNKNOWN";
    actualOpCode: number;
    body: Uint8Array;
    constructor(values: Pick<OpUnknown, 'actualOpCode' | 'body'>);
}
export declare class FullMessage {
    header: MessageHeader;
    contents: OpContents;
    constructor(header: MessageHeader, contents: OpContents);
}
export declare function ParseMessage(data: Uint8Array): Promise<{
    msg: FullMessage;
    readBytes: number;
} | {
    needBytes: number;
}>;
export {};
