"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Proxy = exports.ConnectionPair = void 0;
const net_1 = __importDefault(require("net"));
const parse_stream_1 = require("./parse-stream");
const events_1 = require("events");
class ConnectionPair extends events_1.EventEmitter {
    constructor(info) {
        super();
        this.id = info.id;
        this.incoming = info.incoming;
    }
    toJSON() {
        return { id: this.id, incoming: this.incoming };
    }
}
exports.ConnectionPair = ConnectionPair;
class Proxy extends events_1.EventEmitter {
    constructor(target) {
        super();
        this.connId = 0;
        this.srv = net_1.default.createServer();
        this.srv.on('connection', (conn1) => {
            const conn2 = net_1.default.createConnection(target);
            const conn1reader = new parse_stream_1.WireProtocolParser();
            const conn2reader = new parse_stream_1.WireProtocolParser();
            const cp = new ConnectionPair({
                id: this.connId++,
                incoming: `${conn1.remoteAddress}:${conn1.remotePort}`
            });
            conn1.pipe(conn2);
            conn2.pipe(conn1);
            conn1.pipe(conn1reader);
            conn2.pipe(conn2reader);
            conn1.on('close', () => {
                cp.emit('connectionEnded', 'outgoing');
                conn2.destroy();
                conn1reader.destroy();
            });
            conn2.on('close', () => {
                cp.emit('connectionEnded', 'incoming');
                conn1.destroy();
                conn2reader.destroy();
            });
            conn1.on('error', (err) => {
                cp.emit('connectionError', 'outgoing', err);
            });
            conn2.on('error', (err) => {
                cp.emit('connectionError', 'incoming', err);
            });
            conn1reader.on('message', (msg) => {
                cp.emit('message', 'outgoing', msg);
            });
            conn2reader.on('message', (msg) => {
                cp.emit('message', 'incoming', msg);
            });
            conn1reader.on('error', (err) => {
                cp.emit('parseError', 'outgoing', err);
            });
            conn2reader.on('error', (err) => {
                cp.emit('parseError', 'incoming', err);
            });
            this.emit('newConnection', cp);
        });
    }
    async listen(args) {
        this.srv.listen(args);
        await (0, events_1.once)(this.srv, 'listening');
    }
    address() {
        return this.srv.address();
    }
    async close() {
        await new Promise((resolve, reject) => this.srv.close((err) => (err ? reject(err) : resolve())));
    }
}
exports.Proxy = Proxy;
//# sourceMappingURL=proxy.js.map